@@ -36,6 +36,11 @@
 #include <QFile>
 QTEST_MAIN(KLdapTest)
 
+KLdapTest::KLdapTest(QObject *parent)
+    : QObject(parent)
+{
+}
+
 void KLdapTest::initTestCase()
 {
     /*
@@ -44,9 +49,6 @@ void KLdapTest::initTestCase()
       The specified server should not be a production server in case we break anything here.
       You have been warned!
     */
-    m_search = nullptr;
-    m_model = nullptr;
-
     QString filename(QStringLiteral("testurl.txt"));
     QFile file(filename);
     if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
@@ -54,8 +56,6 @@ void KLdapTest::initTestCase()
         stream >> m_url;
         file.close();
     }
-//    else
-//        QCOMPARE( 0, 1 );
 
     m_search = new LdapSearch;
 
@@ -67,7 +67,6 @@ void KLdapTest::testBer()
 {
     Ber ber1, ber2, ber3, ber4, ber5, ber6, ber7;
     Ber bber;
-    QByteArray flat;
 
     int ainteger;
     QByteArray aoctetString1, aoctetString2, aoctetString3;
@@ -153,15 +152,40 @@ void KLdapTest::testLdapUrl()
                              "dc=kde,dc=org?cn,mail?sub?(objectClass=*)?x-dir=base"));
     url.parseQuery();
 
-    QCOMPARE(url.userName(), QString::fromLatin1("cn=manager,dc=kde,dc=org"));
-    QCOMPARE(url.password(), QString::fromLatin1("password"));
+    QCOMPARE(url.userName(), QStringLiteral("cn=manager,dc=kde,dc=org"));
+    QCOMPARE(url.password(), QStringLiteral("password"));
     QCOMPARE(url.dn(), LdapDN(QStringLiteral("dc=kde,dc=org")));
     QCOMPARE(url.scope(), LdapUrl::Sub);
-    QCOMPARE(url.attributes().at(0), QString::fromLatin1("cn"));
-    QCOMPARE(url.attributes().at(1), QString::fromLatin1("mail"));
-    QCOMPARE(url.filter(), QString::fromLatin1("(objectClass=*)"));
-    QCOMPARE(url.extension(QString::fromLatin1("x-dir"), critical),
-             QString::fromLatin1("base"));
+    QCOMPARE(url.attributes().at(0), QStringLiteral("cn"));
+    QCOMPARE(url.attributes().at(1), QStringLiteral("mail"));
+    QCOMPARE(url.filter(), QStringLiteral("(objectClass=*)"));
+    QCOMPARE(url.extension(QStringLiteral("x-dir"), critical),
+             QStringLiteral("base"));
+    QCOMPARE(url.query(), QStringLiteral("?cn,mail?sub??x-dir=base"));
+    // For some reason the code removes the filter if it's (objectClass=*)...
+    QCOMPARE(url.toString(), QStringLiteral("ldap://cn=manager,dc=kde,dc=org:password@localhost:3999/"
+                                            "dc=kde,dc=org??cn,mail?sub??x-dir=base"));
+
+    // Now set a different filter
+    url.setFilter(QStringLiteral("(objectclass=person)"));
+    QCOMPARE(url.toDisplayString(), QStringLiteral("ldap://cn=manager,dc=kde,dc=org@localhost:3999/"
+                                                   "dc=kde,dc=org??cn,mail?sub?%28objectclass%3Dperson%29?x-dir=base"));
+    QCOMPARE(url.filter(), QStringLiteral("(objectclass=person)"));
+
+    // And now a filter with non-ascii letters
+    url.setFilter(QString::fromUtf8("(givenName=Valérie *)"));
+    QCOMPARE(url.toDisplayString(), QString::fromUtf8("ldap://cn=manager,dc=kde,dc=org@localhost:3999/"
+                                                      "dc=kde,dc=org??cn,mail?sub?%28givenName%3DValérie %2A%29?x-dir=base"));
+    QCOMPARE(url.filter(), QStringLiteral("(givenName=Valérie *)"));
+
+    // Test roundtrip via QUrl, as happens when sending it to kio_ldap
+    const QUrl qurl(url);
+    QCOMPARE(qurl.toDisplayString(), url.toDisplayString());
+
+    const LdapUrl kiourl(qurl);
+    QCOMPARE(kiourl.toString(), url.toString());
+    QCOMPARE(kiourl.toDisplayString(), url.toDisplayString());
+    QCOMPARE(kiourl.filter(), QStringLiteral("(givenName=Valérie *)"));
 }
 
 void KLdapTest::testLdapConnection()
@@ -193,10 +217,10 @@ void KLdapTest::testLdapSearch()
     LdapUrl url;
     url.setUrl(m_url);
     url.parseQuery();
-    connect(m_search, SIGNAL(result(KLDAP::LdapSearch*)),
-            this, SLOT(searchResult(KLDAP::LdapSearch*)));
-    connect(m_search, SIGNAL(data(KLDAP::LdapSearch*,KLDAP::LdapObject)),
-            this, SLOT(searchData(KLDAP::LdapSearch*,KLDAP::LdapObject)));
+    connect(m_search, &LdapSearch::result,
+            this, &KLdapTest::searchResult);
+    connect(m_search, &LdapSearch::data,
+            this, &KLdapTest::searchData);
     bool success = m_search->search(url);
     while (QCoreApplication::hasPendingEvents()) {
         qApp->processEvents();
@@ -282,19 +306,19 @@ void KLdapTest::testLdapModel()
              "/dc=kde,dc=org?cn,mail?sub?(objectClass=*)?x-dir=base");
   url.parseQuery();
 
-  QCOMPARE( url.user(), QString::fromLatin1("cn=manager,dc=kde,dc=org") );
-  QCOMPARE( url.password(), QString::fromLatin1("password") );
-  QCOMPARE( url.dn(), QString::fromLatin1("dc=kde,dc=org") );
+  QCOMPARE( url.user(), QStringLiteral("cn=manager,dc=kde,dc=org") );
+  QCOMPARE( url.password(), QStringLiteral("password") );
+  QCOMPARE( url.dn(), QStringLiteral("dc=kde,dc=org") );
   QCOMPARE( url.scope(), LdapUrl::Sub );
-  QCOMPARE( url.attributes().at(0), QString::fromLatin1("cn") );
-  QCOMPARE( url.attributes().at(1), QString::fromLatin1("mail") );
-  QCOMPARE( url.filter(), QString::fromLatin1("(objectClass=*)") );
-  QCOMPARE( url.extension(QString::fromLatin1("x-dir"), critical), QString::fromLatin1("base") );
+  QCOMPARE( url.attributes().at(0), QStringLiteral("cn") );
+  QCOMPARE( url.attributes().at(1), QStringLiteral("mail") );
+  QCOMPARE( url.filter(), QStringLiteral("(objectClass=*)") );
+  QCOMPARE( url.extension(QStringLiteral("x-dir"), critical), QStringLiteral("base") );
 
   url.setDn("ou=People,dc=kde,dc=org");
-  QCOMPARE( url.dn(), QString::fromLatin1("ou=People,dc=kde,dc=org") );
+  QCOMPARE( url.dn(), QStringLiteral("ou=People,dc=kde,dc=org") );
   url.setDn("/ou=People,dc=kde,dc=org");
-  QCOMPARE( url.dn(), QString::fromLatin1("ou=People,dc=kde,dc=org") );
+  QCOMPARE( url.dn(), QStringLiteral("ou=People,dc=kde,dc=org") );
 
   LdapServer server;
 //  url.setUrl("ldaps://cn=manager,dc=kde,dc=org:passwor@localhost:3999/"
@@ -303,20 +327,20 @@ url.setUrl("ldaps://cn=manager,dc=kde,dc=org:password@localhost:3999/"
            "dc=kde,dc=org??base??x-timelimit=5");
 url.parseQuery();
 server.setUrl( url );
-QCOMPARE( url.query(), QString::fromLatin1("??base??x-timelimit=5") );
+QCOMPARE( url.query(), QStringLiteral("??base??x-timelimit=5") );
 QCOMPARE( url.url(), server.url().url() );
 
 LdapControl c1;
-c1.setControl( QString::fromLatin1("1.2.3.4.5.6"), QByteArray("abcdefg"), true );
+c1.setControl( QStringLiteral("1.2.3.4.5.6"), QByteArray("abcdefg"), true );
 //test copy constructor
 LdapControl c2(c1);
-QCOMPARE( c2.oid(), QString::fromLatin1("1.2.3.4.5.6") );
+QCOMPARE( c2.oid(), QStringLiteral("1.2.3.4.5.6") );
 QCOMPARE( c2.value(), QByteArray("abcdefg") );
 QCOMPARE( c2.critical(), true );
 //test assignment operator
 LdapControl c3;
 c3 = c1;
-QCOMPARE( c3.oid(), QString::fromLatin1("1.2.3.4.5.6") );
+QCOMPARE( c3.oid(), QStringLiteral("1.2.3.4.5.6") );
 QCOMPARE( c3.value(), QByteArray("abcdefg") );
 QCOMPARE( c3.critical(), true );
 */
